##################### generated by xml-casa (v2) from uvcontsub2021.xml #############
##################### e85786bae84e96de6e7876daceb53cd8 ##############################
from __future__ import absolute_import
import numpy
from casatools.typecheck import CasaValidator as _val_ctor
_pc = _val_ctor( )
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from .private.task_uvcontsub2021 import uvcontsub2021 as _uvcontsub2021_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from casatasks.private.task_logging import except_log as _except_log

class _uvcontsub2021:
    """
    uvcontsub2021 ---- continuum subtraction in the uv plane

    --------- parameter descriptions ---------------------------------------------

    vis         Name of input visibility file.
                Default: '', must be specified
                
                   Example: vis='ngc5921.ms'
    outputvis   Name of output visibility file
                Default: '', must be specified
                
                Example: outputvis='ngc5921_contsub.ms'
                
                If an MS with the output name already exists, it
                will not be overwritten and the task stops with
                an error.
    field       Select field using field id(s) or field name(s)
                Default: '' (all fields)
                
                Use 'listobs' to obtain the list of id's or
                names. If field string is a non-negative integer,
                it is assumed a field index,  otherwise, it is
                assumed a field name.
                
                   Examples:
                   field='0~2'; field ids 0,1,2
                   field='0,4,5~7'; field ids 0,4,5,6,7
                   field='3C286,3C295'; field named 3C286 and
                   3C295
                   field = '3,4C*'; field id 3, all names
                   starting with 4C
    spw         Select spectral window/channels
                          Default: '' (meaning all spectral windows and channels)
                
                             Examples:
                             spw='0~2,4'; spectral windows 0,1,2,4 (all channels)
                             spw='<2';  spectral windows less than 2 (i.e. 0,1)
                             spw='0:5~61'; spw 0, channels 5 to 61
                             spw='0,10,3:3~45'; spw 0,10 all channels, spw
                             3 - chans 3 to 45.
                             spw='0~2:2~6'; spw 0,1,2 with channels 2
                             through 6 in each.
                             spw = '*:3~64'  channels 3 through 64 for all sp id's
                             spw = ' :3~64' will NOT work.
    scan        Select scans by scan numbers
                Default: '' = all
                
                Examples:
                scan='3'
                scan='2~23'
                scan=''4,7,9,12'
    intent      Select observing intent
                Default: '' (no selection by intent)
                
                Example: intent='*BANDPASS*'  (selects data
                         labelled with BANDPASS intent)
    array       Select (sub)array(s) by array ID number
                Default: '' (all)
    observation Select by observation ID(s)
                Subparameter of selectdata=True
                Default: '' = all
                
                    Example: observation='0~2,4'
    datacolumn  Which data column(s) to use for processing
                (case-insensitive).
                Default: 'data'
                Options: 'data', 'model', 'corrected',
                'float_data', 'lag_data',
                
                Example: datacolumn='corrected'
    fitspw      Spectral window:channel selections for fitting,
                per field. The continuum will be fitted using as input the
                selected channels, which can be considered the line-free
                channels.  Must be a list with up to as many elements as
                fields have been selected from the input MS. Each element of
                the list specifies a pair [field-ID(s), spw:channel-selection].
                
                If a field or SPW is not included, the default (all channels)
                is assumed.
                
                             Default: [] (all channels for all SPWs for all fields)
                             Examples:
                             (assuming: 4 SPWs, with IDs 17, 19, 21, 23.
                                        5 fields, with IDs 0, 1, 4, 5, 6)
                             [
                              ['0', ''],
                              ['1', '17:8~120,19:7~100;21~119,23'],
                              ['4,5,6', ',19:7~119,21,23']
                             ]
                             (field 0: fit all SPWs and channels (same as not giving it),
                              only some channel ranges for field 1 (all for SPW 23),
                              only some channel ranges for fields 4,5,6 in SPW 19,
                              all channels for SPWs 21 and 23, and don't fit SPWs 17)
                
                In this example, the empty string given for field 0 means that
                all the channels are fitted (and subtracted) in all the
                SPWs. For the corner case where for a given field no channels
                should be fitted in any SPW, the spw:channels string can be
                set to 'NONE'.
                
                This selection of SPWs and channels uses a syntax similar to
                the spw parameter. But here the SPWs and channels are used as
                a mask applied on the input data only for fitting
                purposes. The channels that are not selected are masked out
                and not used as input by the fitting algorithm. This is
                different from the spw parameter, which selects the data that
                is included in the output.
    fitmethod   Choose fitting methods
                Default: 'gsl'
                Options: 'gsl', 'casacore'
                
                Example: fitmethod='gsl'
    fitorder    Polynomial order for the fits
    writemodel  Write fitted model into the MODEL column of the
                output MeasurementSet.
                
                             Default: False
                             Options: True|False
                
                             By default the task creates an MS with the
                             continuum subtracted data in the DATA column. If
                             this option is enabled, the task will also write
                             the fitted model data into the MODEL column of the
                             output MS.

    --------- examples -----------------------------------------------------------

    
    For more information, see the task pages of uvcontsub in CASA Docs:
    https://casadocs.readthedocs.io
    


    """

    _info_group_ = """manipulation"""
    _info_desc_ = """continuum subtraction in the uv plane"""

    def __call__( self, vis='', outputvis='', field='', spw='', scan='', intent='', array='', observation='', datacolumn='data', fitspw='', fitmethod='gsl', fitorder=int(0), writemodel=False ):
        schema = {'vis': {'type': 'cReqPath', 'coerce': _coerce.expand_path}, 'outputvis': {'type': 'cStr', 'coerce': _coerce.to_str}, 'field': {'type': 'cStr', 'coerce': _coerce.to_str}, 'spw': {'type': 'cStr', 'coerce': _coerce.to_str}, 'scan': {'type': 'cStr', 'coerce': _coerce.to_str}, 'intent': {'type': 'cStr', 'coerce': _coerce.to_str}, 'array': {'type': 'cStr', 'coerce': _coerce.to_str}, 'observation': {'type': 'cStr', 'coerce': _coerce.to_str}, 'datacolumn': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'DATA', 'model', 'corrected', 'LAG_DATA', 'lag_data', 'FLOAT_DATA', 'CORRECTED', 'float_data', 'MODEL', 'data' ]}, 'fitspw': {'anyof': [{'type': 'cStr', 'coerce': _coerce.to_str}, {'type': 'cVariant', 'coerce': [_coerce.to_variant]}]}, 'fitmethod': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'gsl', 'casacore', 'GSL', 'CASACORE' ]}, 'fitorder': {'type': 'cInt'}, 'writemodel': {'type': 'cBool'}}
        doc = {'vis': vis, 'outputvis': outputvis, 'field': field, 'spw': spw, 'scan': scan, 'intent': intent, 'array': array, 'observation': observation, 'datacolumn': datacolumn, 'fitspw': fitspw, 'fitmethod': fitmethod, 'fitorder': fitorder, 'writemodel': writemodel}
        assert _pc.validate(doc,schema), create_error_string(_pc.errors)
        _logging_state_ = _start_log( 'uvcontsub2021', [ 'vis=' + repr(_pc.document['vis']), 'outputvis=' + repr(_pc.document['outputvis']), 'field=' + repr(_pc.document['field']), 'spw=' + repr(_pc.document['spw']), 'scan=' + repr(_pc.document['scan']), 'intent=' + repr(_pc.document['intent']), 'array=' + repr(_pc.document['array']), 'observation=' + repr(_pc.document['observation']), 'datacolumn=' + repr(_pc.document['datacolumn']), 'fitspw=' + repr(_pc.document['fitspw']), 'fitmethod=' + repr(_pc.document['fitmethod']), 'fitorder=' + repr(_pc.document['fitorder']), 'writemodel=' + repr(_pc.document['writemodel']) ] )
        task_result = None
        try:
            task_result = _uvcontsub2021_t( _pc.document['vis'], _pc.document['outputvis'], _pc.document['field'], _pc.document['spw'], _pc.document['scan'], _pc.document['intent'], _pc.document['array'], _pc.document['observation'], _pc.document['datacolumn'], _pc.document['fitspw'], _pc.document['fitmethod'], _pc.document['fitorder'], _pc.document['writemodel'] )
        except Exception as exc:
            _except_log('uvcontsub2021', exc)
            raise
        finally:
            task_result = _end_log( _logging_state_, 'uvcontsub2021', task_result )
        return task_result

uvcontsub2021 = _uvcontsub2021( )

